@@ -7,7 +7,7 @@ Panda::XS - useful features and typemaps for XS modules.
 
 =cut
 
-our $VERSION = '1.1.2';
+our $VERSION = '1.2.0';
 require Panda::XSLoader;
 Panda::XSLoader::load();
 
@@ -243,7 +243,7 @@ Otherwise, shared pointer won't be released!
 
 =item T_OEXT
 
-=item YOUR_TYPE : T_OEXT([basetype=classname], [nocast=1])
+=item YOUR_TYPE : T_OEXT([basetype=classname], [nocast=1], [on_svdup=funcname])
 
 Extendable object with C pointer attached.
 
@@ -297,6 +297,15 @@ By default $basetype = $type. If $basetype == $type, then casting is not perform
 If set, disables typecasting described in 'basetype' section even if $basetype != $type. Useful for storing objects in wrappers.
 See C<HOW TO> for details.
 
+=item on_svdup
+
+Callback (function) name to call on threaded perls when a new thread is created and your object is about to be copied. Function
+signature is:
+
+    <basetype> <callbackname> (<basetype> arg);
+
+See L<THREADED PERLS SUPPORT> for details.
+
 =back
 
 =over Special C variables for OUTPUT
@@ -1383,10 +1392,28 @@ Removes payload from sv and returns number of payloads removed.
 
 Same as sv_* but operates on sv referenced by a given rv. Doesn't check if a given RV is valid.
 
-=head4 payload_marker_t* sv_payload_marker (const char* class_name)
+=head4 payload_marker_t* sv_payload_marker (const char* key, on_svdup_t svdup_callback = NULL)
 
-Returns a marker for specified key. If no marker exists for that key, creates and returns it.
+Returns a marker for specified key. If no marker exists for that key, creates and returns it. In most cases you should put the 
+result into a static variable to prevent calling this function many times.
 
+C<svdup_callback> is a callback to run on threaded perls when a thread is created. Called for each object and receives magic struct
+attach to an object with current marker. Return value is ignored. Used to clone or do something other with payload attached to this
+magic. Note that you don't need to clone SV* payload, as threaded perl duplicates it by itself.
+
+    typedef int (*on_svdup_t) (pTHX_ MAGIC* mg, CLONE_PARAMS* param);
+
+=head4 template <class T> payload_marker_t SVPayloadMarker::marker
+
+Marker for particular class. In contrast to C<sv_payload_marker> function, no code is executed, so you dont have to save it to a
+static variable. Used internally by T_OEXT typemap class. 
+
+Note that marker C<SVPayloadMarker<MyClass*>::marker> is NOT the same marker as C<sv_payload_marker("MyClass*")>.
+
+=head4 template <class T> payload_marker_t* SVPayloadMarker::get(on_svdup_t svdup_callback)
+
+Returns the same marker as C<&SVPayloadMarker<T>::marker>, but before returning, sets svdup_callback as "duplication" callback for 
+threaded perls.
 
 =head2 SUPER/NEXT METHOD FUNCTIONS
 
@@ -1500,6 +1527,73 @@ Same as sv_* but $target must be reference to perl object.
 Same as sv_* or rv_*, but if $target is ref then payload will be attached/detached from referenced object, 
 otherwise from $target itself.
 
+
+
+=head1 THREADED PERLS SUPPORT
+
+If you don't do anything special, on threaded perls after thread creation you will most likely core dump.
+The reason is that when perl creates a thread it duplicates all perl variables available in current interpreter.
+So that you will likely get 2 perl variables holding a pointer to the case C/C++ object (refcnt won't help - nobody calls C<retain()>).
+When these objects are destroyed, double C<delete> might occur on the same C object. That's bad.
+
+You have 4 options to handle that:
+
+=over
+
+=item Don't support thread creation (however don't core dump).
+
+To do this, just add C<CLONE_SKIP> method to your class and return
+a true value from it. In this case perl won't clone SV variable holding your object - it will place an unblessed reference to undef
+instead. Of course users must know about this behaviour, because they won't be able to use objects of your class created before
+new thread. However they can still create new objects from new thread and everything will be ok.
+
+This is the only way for you if you use T_OPTR. T_OEXT doesn't have this limitation.
+
+=item Support thread creation by cloning your C++ object (for example, if it's thread-unsafe or doesn't have a refcounter)
+
+To do this, don't define C<CLONE_SKIP> method, instead set C<on_svdup> parameter of T_OEXT class to '@clone'.
+In this case when a thread creates, method C<clone()> will be called on your C++ object and this new object is stored in duplicated
+variable. Of course you must have a C<clone()> method with no args in your class. It must be virtual if you support inheritance.
+For example:
+
+    T_MYCLASS : T_OEXT(basetype=MyClass*, on_svdup=@clone)
+
+=item Support thread creation by incrementing refcnt on your C++ object (for thread-safe objects with refcounters).
+
+To do this, don't define C<CLONE_SKIP> method, instead set C<on_svdup> parameter of T_OEXT class to '@retain'.
+In this case when a thread creates, method C<retain()> will be called on your C++ object and the same object is stored in duplicated
+variable. Of course you must have a C<retain()> method with no args in your class (if you subclass panda::RefCounted, you do).
+
+For example:
+
+    T_MYCLASS : T_OEXT(basetype=MyClass*, on_svdup=@retain)
+
+=item Implement custom logic
+
+To do this, don't define C<CLONE_SKIP> method, instead set C<on_svdup> parameter of T_OEXT class to the name of your callback which
+will handle thread creation for objects of your class. Callback must have the following signature:
+
+    <basetype> <callbackname> (<basetype> arg);
+    
+Callback will be passed current C object. Callback shall return a C object to be set to the duplicated perl variable
+(it may be a new one or the same object). Keep in mind that if your module shares API for other modules, this callback MUST be
+externally visible, because when someone uses your class as a return type from XS functions, typemap will insert code, containing
+your callback name into those XS functions. Don't forget to specify it with full qualified name for the same reason. Example:
+
+    # in .h
+    namespace my {
+        MyClass* myclass_onsvdup (MyClass* old);
+    }
+    # in .cc
+    MyClass* my::myclass_onsvdup (MyClass* old) {
+        return new MyClass(old->data);
+    }
+    
+    # in typemap
+    T_MYCLASS : T_OEXT(basetype=MyClass*, on_svdup=my::myclass_onsvdup)
+    
+=over
+
 =head1 CAVEATS
 
 If module A binary depends on module B and module B updates and becomes binary incompatible, undefined behaviour may happen.
