@@ -12,9 +12,11 @@ std::map<std::string, payload_marker_t> sv_class_markers;
 
 const int CVf_NEXT_WRAPPER_CREATED = 0x10000000;
 
-payload_marker_t* sv_payload_marker (const char* class_name) {
+payload_marker_t* sv_payload_marker (const char* class_name, on_svdup_t dup_callback) {
     if (!class_name[0]) return &sv_payload_default_marker;
-    return &sv_class_markers[class_name];
+    payload_marker_t* marker = &sv_class_markers[class_name];
+    if (!marker->svt_dup && dup_callback) marker->svt_dup = dup_callback;
+    return marker;
 }
 
 static SV* _next_create_wrapper (pTHX_ CV* cv, next_t type) {
