@@ -51,6 +51,26 @@ typedef IO OIO;
 #define XS_AV_ITER_NE(av,code) XS_AV_ITER(av,{if(!elem) continue; code})
 #define XS_AV_ITER_NU(av,code) XS_AV_ITER(av,{if(!elem || !SvOK(elem)) continue; code})
 
+// Threaded-perl helpers
+
+#ifndef THREAD_LOCAL
+#  ifdef _MSC_VER
+#    define THREAD_LOCAL __declspec( thread )
+#  else
+#    define THREAD_LOCAL __thread
+#  endif
+#endif
+
+#ifdef aTHX // define class member helpers for storing perl interpreter
+#  define mTHX      pTHX;
+#  define mTHXa(a)  aTHX(a)
+#  define mTHXa_(a) aTHX(a),
+#else
+#  define mTHX
+#  define mTHXa(a)
+#  define mTHXa_(a)
+#endif
+
 namespace xs {
 
 enum next_t {
@@ -59,18 +79,34 @@ enum next_t {
     NEXT_MAYBE  = 2
 };
 
+//THREAD_LOCAL PerlInterpreter* my_perl;
+
+typedef int (*on_svdup_t) (pTHX_ MAGIC* mg, CLONE_PARAMS* param);
+
 typedef MGVTBL payload_marker_t;
 extern payload_marker_t sv_payload_default_marker;
-payload_marker_t* sv_payload_marker (const char* class_name);
+payload_marker_t* sv_payload_marker (const char* class_name, on_svdup_t svdup_callback = NULL);
+
+template <class T>
+struct SVPayloadMarker {
+    static payload_marker_t marker;
+    static payload_marker_t* get (on_svdup_t dup_callback = NULL) {
+        if (dup_callback) marker.svt_dup = dup_callback;
+        return &marker;
+    }
+};
+template <class T> payload_marker_t SVPayloadMarker<T>::marker;
 
 inline void sv_payload_attach (pTHX_ SV* sv, void* ptr, const payload_marker_t* marker = &sv_payload_default_marker) {
-    sv_magicext(sv, NULL, PERL_MAGIC_ext, marker ? marker : &sv_payload_default_marker, (const char*) ptr, 0);
+    MAGIC* mg = sv_magicext(sv, NULL, PERL_MAGIC_ext, marker ? marker : &sv_payload_default_marker, (const char*) ptr, 0);
+    if (marker->svt_dup) mg->mg_flags |= MGf_DUP;
     SvRMAGICAL_off(sv); // remove unnecessary perfomance overheat
 }
 
 inline void sv_payload_attach (pTHX_ SV* sv, void* ptr, SV* obj, const payload_marker_t* marker = &sv_payload_default_marker) {
     MAGIC* mg = sv_magicext(sv, obj, PERL_MAGIC_ext, marker ? marker : &sv_payload_default_marker, (const char*) ptr, 0);
     mg->mg_flags |= MGf_REFCOUNTED;
+    if (marker->svt_dup) mg->mg_flags |= MGf_DUP;
     SvRMAGICAL_off(sv); // remove unnecessary perfomance overheat
 }
 
@@ -128,139 +164,11 @@ inline int rv_payload_detach (pTHX_ const SV* rv, payload_marker_t* marker = NUL
     return sv_payload_detach(aTHX_ SvRV(rv), marker);
 }
 
-inline SV* _typemap_out_oref (pTHX_ SV* var, HV* CLASS) {
-    return var ? sv_bless(newRV_noinc(var), CLASS) : &PL_sv_undef;
-}
-inline SV* _typemap_out_oref (pTHX_ SV* var, const char* CLASS) {
-    return _typemap_out_oref(aTHX_ var, gv_stashpvn(CLASS, strlen(CLASS), GV_ADD));
-}
-inline SV* _typemap_out_oref (pTHX_ SV* var, SV* CLASS) {
-    return _typemap_out_oref(aTHX_ var, gv_stashsv(CLASS, GV_ADD));
-}
-
-inline SV* _typemap_out_optr (pTHX_ void* var, HV* CLASS) {
-    return var ? sv_bless(newRV_noinc(newSViv((IV)var)), CLASS) : &PL_sv_undef;
-}
-inline SV* _typemap_out_optr (pTHX_ void* var, const char* CLASS) {
-    return _typemap_out_optr(aTHX_ var, gv_stashpvn(CLASS, strlen(CLASS), GV_ADD));
-}
-inline SV* _typemap_out_optr (pTHX_ void* var, SV* CLASS) {
-    return _typemap_out_optr(aTHX_ var, gv_stashsv(CLASS, GV_ADD));
-}
-
-template <class T, typename C>
-inline SV* _typemap_out_optr (pTHX_ const panda::shared_ptr<T, true> &sp, C CLASS) {
-     sp->retain();
-     return _typemap_out_optr(aTHX_ sp.get(), CLASS);
-}
-
-template <class T, typename C>
-inline SV* _typemap_out_optr (pTHX_ const panda::shared_ptr<T, false> &sp, C CLASS) {
-    return _typemap_out_optr(aTHX_ new panda::shared_ptr<T>(sp), CLASS);
-}
-
-#ifdef CPP11X
-template <class T, typename C>
-inline SV* _typemap_out_optr (pTHX_ const std::shared_ptr<T> &sp, C CLASS) {
-    return _typemap_out_optr(aTHX_ new std::shared_ptr<T>(sp), CLASS);
-}
-#endif
-
-SV* _typemap_out_oext (pTHX_ SV* self, void* var, HV* CLASS, payload_marker_t* marker = NULL);
-SV* _typemap_out_oext (pTHX_ SV* self, void* var, SV* CLASS, payload_marker_t* marker = NULL);
-SV* _typemap_out_oext (pTHX_ SV* self, void* var, const char* CLASS, payload_marker_t* marker = NULL);
-
-template <class T, typename C>
-inline SV* _typemap_out_oext (pTHX_ SV* self, const panda::shared_ptr<T, true> &sp, C CLASS, payload_marker_t* marker = NULL) {
-     sp->retain();
-     return _typemap_out_oext(aTHX_ self, sp.get(), CLASS, marker);
-}
-
-template <class T, typename C>
-inline SV* _typemap_out_oext (pTHX_ SV* self, const panda::shared_ptr<T, false> &sp, C CLASS, payload_marker_t* marker = NULL) {
-    return _typemap_out_oext(aTHX_ self, new panda::shared_ptr<T>(sp), CLASS, marker);
-}
-
-#ifdef CPP11X
-template <class T, typename C>
-inline SV* _typemap_out_oext (pTHX_ SV* self, const std::shared_ptr<T> &sp, C CLASS, payload_marker_t* marker = NULL) {
-    return _typemap_out_oext(aTHX_ self, new std::shared_ptr<T>(sp), CLASS, marker);
-}
-#endif
-
-template <class T>
-inline void _typemap_in_optr (pTHX_ SV* arg, T* varptr) {
-    if (sv_isobject(arg)) {
-        SV* obj = SvRV(arg);
-        if (SvIOK(obj)) {
-            *varptr = static_cast<T>((void*)SvIVX(obj));
-            return;
-        }
-    }
-    *varptr = NULL;
-}
-
-template <class T>
-inline void _typemap_in_optr (pTHX_ SV* arg, panda::shared_ptr<T,true>* sptr, bool destroy = false) {
-    void* ptr;
-    _typemap_in_optr(aTHX_ arg, &ptr);
-    *sptr = static_cast<T*>(ptr);
-    if (destroy) static_cast<T*>(ptr)->release();
-}
-
-template <class T>
-inline void _typemap_in_optr (pTHX_ SV* arg, panda::shared_ptr<T,false>* sptr, bool destroy = false) {
-    void* ptr;
-    _typemap_in_optr(aTHX_ arg, &ptr);
-    *sptr = *(static_cast<panda::shared_ptr<T,false>*>(ptr));
-    if (destroy) delete static_cast<panda::shared_ptr<T,false>*>(ptr);
-}
-
-#ifdef CPP11X
-template <class T>
-inline void _typemap_in_optr (pTHX_ SV* arg, std::shared_ptr<T>* sptr, bool destroy = false) {
-    void* ptr;
-    _typemap_in_optr(aTHX_ arg, &ptr);
-    *sptr = *(static_cast<std::shared_ptr<T>*>(ptr));
-    if (destroy) delete static_cast<std::shared_ptr<T>*>(ptr);
-}
-#endif
-
-template <class T>
-inline void _typemap_in_oext (pTHX_ SV* arg, T* varptr, payload_marker_t* marker = NULL) {
-    if (SvROK(arg)) *varptr = static_cast<T>(rv_payload(aTHX_ arg, marker));
-    else *varptr = NULL;
-}
-
-template <class T>
-inline void _typemap_in_oext (pTHX_ SV* arg, panda::shared_ptr<T,true>* sptr, payload_marker_t* marker = NULL, bool destroy = false) {
-    void* ptr;
-    _typemap_in_oext(aTHX_ arg, &ptr, marker);
-    *sptr = static_cast<T*>(ptr);
-    if (destroy) static_cast<T*>(ptr)->release();
-}
-
-template <class T>
-inline void _typemap_in_oext (pTHX_ SV* arg, panda::shared_ptr<T,false>* sptr, payload_marker_t* marker = NULL, bool destroy = false) {
-    void* ptr;
-    _typemap_in_oext(aTHX_ arg, &ptr, marker);
-    *sptr = *(static_cast<panda::shared_ptr<T,false>*>(ptr));
-    if (destroy) delete static_cast<panda::shared_ptr<T,false>*>(ptr);
-}
-
-#ifdef CPP11X
-template <class T>
-inline void _typemap_in_oext (pTHX_ SV* arg, std::shared_ptr<T>* sptr, payload_marker_t* marker = NULL, bool destroy = false) {
-    void* ptr;
-    _typemap_in_oext(aTHX_ arg, &ptr, marker);
-    *sptr = *(static_cast<std::shared_ptr<T>*>(ptr));
-    if (destroy) delete static_cast<std::shared_ptr<T>*>(ptr);
-}
-#endif
-
 SV* call_next (pTHX_ CV* cv, SV** args, I32 items, next_t type, I32 flags = 0);
 inline SV* call_super       (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0) { return call_next(aTHX_ cv, args, items, NEXT_SUPER, flags); }
 inline SV* call_next_method (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0) { return call_next(aTHX_ cv, args, items, NEXT_METHOD, flags); }
 inline SV* call_next_maybe  (pTHX_ CV* cv, SV** args, I32 items, I32 flags = 0) { return call_next(aTHX_ cv, args, items, NEXT_MAYBE, flags); }
 
 };
+
+#include <xs/xs-private.h>
