@@ -40,6 +40,7 @@ class MyBase {
     public:
     int val;
     MyBase (int arg) : val(arg) {}
+    virtual MyBase* clone () { return new MyBase(val); }
     virtual ~MyBase () { dcnt++; }
 };
 
@@ -47,9 +48,17 @@ class MyChild : public MyBase {
     public:
     int val2;
     MyChild (int arg1, int arg2) : val2(arg2), MyBase(arg1) {}
+    MyBase* clone () { return new MyChild(val, val2); }
     virtual ~MyChild () { dcnt++; }
 };
 
+class MyThreadSafe : public RefCounted {
+    public:
+    int val;
+    MyThreadSafe (int arg) : val(arg) {}
+    virtual ~MyThreadSafe () { dcnt++; }
+};
+
 class MyOther {
     public:
     int val;
@@ -125,6 +134,10 @@ class MyClassChild : public MyClass {
     }
 };
 
+inline MyBase* pxs_mybase_dup (pTHX_ MyBase* obj) {
+    return obj->clone();
+}
+
 typedef panda::shared_ptr<MyRefCounted> MyRefCountedSP;
 typedef panda::shared_ptr<MyRefCountedChild> MyRefCountedChildSP;
 typedef panda::shared_ptr<MyClass> MyClassSP;
